@@ -1,5 +1,6 @@
+/* SPDX-License-Identifier: GPL-2.0 */
 /*
- * Copyright (C) 2005-2017 Junjiro R. Okajima
+ * Copyright (C) 2005-2018 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +37,7 @@ struct au_hnotify {
 	struct fsnotify_mark		hn_mark;
 #endif
 	struct inode		*hn_aufs_inode;	/* no get/put */
+	struct rcu_head		rcu;
 #endif
 } ____cacheline_aligned_in_smp;
 
@@ -76,9 +78,10 @@ struct au_iinfo {
 };
 
 struct au_icntnr {
-	struct au_iinfo iinfo;
-	struct inode vfs_inode;
-	struct hlist_node plink;
+	struct au_iinfo		iinfo;
+	struct inode		vfs_inode;
+	struct hlist_bl_node	plink;
+	struct rcu_head		rcu;
 } ____cacheline_aligned_in_smp;
 
 /* au_pin flags */
@@ -394,10 +397,9 @@ AuRWLockFuncs(new_child, NEW_CHILD);
 #undef AuWriteLockFunc
 #undef AuRWLockFuncs
 
-/*
- * ii_read_unlock, ii_write_unlock, ii_downgrade_lock
- */
-AuSimpleUnlockRwsemFuncs(ii, struct inode *i, &au_ii(i)->ii_rwsem);
+#define ii_read_unlock(i)	au_rw_read_unlock(&au_ii(i)->ii_rwsem)
+#define ii_write_unlock(i)	au_rw_write_unlock(&au_ii(i)->ii_rwsem)
+#define ii_downgrade_lock(i)	au_rw_dgrade_lock(&au_ii(i)->ii_rwsem)
 
 #define IiMustNoWaiters(i)	AuRwMustNoWaiters(&au_ii(i)->ii_rwsem)
 #define IiMustAnyLock(i)	AuRwMustAnyLock(&au_ii(i)->ii_rwsem)
@@ -676,6 +678,16 @@ static inline void au_hn_inode_lock_nested(struct au_hinode *hdir,
 	au_hn_suspend(hdir);
 }
 
+#if 0 /* unused */
+#include "vfsub.h"
+static inline void au_hn_inode_lock_shared_nested(struct au_hinode *hdir,
+						  unsigned int sc)
+{
+	vfsub_inode_lock_shared_nested(hdir->hi_inode, sc);
+	au_hn_suspend(hdir);
+}
+#endif
+
 static inline void au_hn_inode_unlock(struct au_hinode *hdir)
 {
 	au_hn_resume(hdir);
